@@ -4,7 +4,7 @@ import logging
 
 from .alert import Alert, get_alerts
 from .entity import Entity
-from .helpers import to_unix_time
+from .helpers import to_time_query_param
 
 logger = logging.getLogger(__name__)
 
@@ -183,7 +183,7 @@ class Device(Entity):
 
         return response
 
-    def data(self, start=None, end=None, topics=None, direction="forward", api_call_kwargs={}, **params):
+    def data(self, start=None, end=None, topics=None, direction="forward", loop=False, api_call_kwargs={}, **params):
         """iter: Fetches data from the robot for a specified interval and returns an iterator
         to page through the data. Each next() call to the iterator will return 1 message, and
         data will be fetched from the server as necessary until the end has been reached.
@@ -193,15 +193,26 @@ class Device(Entity):
             end (float): UNIX time of the end of the desired data interval.
             topics (`list` of `str`): List of topics of interest (in most cases, ROS topics).
             direction (str): "forward" or "backward".
+            loop (bool): If True, will keep fetching data forever between the selected `start` and `end`
+                filters. If you want to loop for live data, then you should remove the end argument.
             api_call_kwargs (dict): Optional keyword arguments for api call.
             **params (dict): Optional query parameters.
         """
+        utc_start = utc_end = None
         if start is not None:
-            params["utc_start"] = to_unix_time(start)
+            utc_start = to_time_query_param(start)
             if end is None:
-                params["utc_end"] = to_unix_time(start) + 60.0
+                # If is not a string, set 1 minute for the end
+                if not isinstance(utc_start, str):
+                    utc_end = to_time_query_param(start) + 60.0
             else:
-                params["utc_end"] = to_unix_time(end)
+                utc_end = to_time_query_param(end)
+
+        if utc_start is not None:
+            params['utc_start'] = utc_start
+
+        if utc_end is not None:
+            params['utc_end'] = utc_end
 
         if topics is not None:
             params["topics"] = json.dumps(topics)
@@ -209,6 +220,12 @@ class Device(Entity):
         params["pagination"] = True
         params["pagination_direction"] = direction
 
+        def _choose_utc_time(a, b):
+            if not a or isinstance(a, str):
+                return b
+            else:
+                return a
+
         while True:
             result = self.api.call(
                 "GET",
@@ -225,31 +242,48 @@ class Device(Entity):
                 result["messages"].sort(key=lambda item: item.get("utc_time", 0.0), reverse=True)
 
             if result is None or len(result["messages"]) == 0:
-                return
+                if loop:
+                    # If no results and we are looping, we should start over
+                    params['utc_start'] = _choose_utc_time(utc_start, result['requested_utc_start'])
+                    params['utc_end'] = _choose_utc_time(utc_end, result['requested_utc_end'])
+                    continue
+                else:
+                    return
 
             logger.debug("fetched %d of %d messages" % (
                 result.get("returned_num_messages", -1), result.get("requested_num_messages", -1)
             ))
 
-            if not result.get("returned_utc_start") or not result.get("returned_utc_end"):
-                return
-
             for item in result.get("messages", {}):
                 logger.debug("item: ", item)
                 yield item
 
+            returned_period = result["returned_utc_end"] - result["returned_utc_start"]
+
             if direction == "forward":
-                params["utc_start"] = result.get("returned_utc_end")
+                params["utc_start"] = result["returned_utc_end"]
                 params["utc_end"] = min(
-                    params["utc_start"] + 1.1 * (result.get("returned_utc_end") - result.get("returned_utc_start")),
-                    end
+                    params['utc_start'] + 1.1 * returned_period,
+                    _choose_utc_time(utc_end, result['requested_utc_end'])
                 )
             elif direction == "backward":
-                params["utc_end"] = result.get("returned_utc_start")
+                params["utc_end"] = result["returned_utc_start"]
                 params["utc_start"] = max(
-                    params["utc_end"] - 1.1 * (result.get("returned_utc_end") - result.get("returned_utc_start")),
-                    start
+                    params['utc_end'] - 1.1 * returned_period,
+                    _choose_utc_time(utc_start, result['requested_utc_start'])
                 )
+            # if start reaches end value, we finish (or start over if loop)
+            if params['utc_start'] == params['utc_end']:
+                if loop:
+                    # if we have set an end, start over, otherwise continue asking
+                    # for new data
+                    if utc_end:
+                        params['utc_start'] = utc_start
+                        params['utc_end'] = utc_end
+                    else:
+                        params.pop('utc_end', None)
+                else:
+                    return
 
     def data_last(self, topics=None, params=None, api_call_kwargs={}):
         """Returns the last message published by the device, as long as it is within the past 1 minute.
