@@ -3,9 +3,11 @@ from .alert import get_alerts
 
 
 class Zone(Entity):
+    pk = 'id'
 
-    def _make_url(self):
-        return "/accounts/{}/zones/{}".format(self._data.get('account_id'), self._id)
+    @property
+    def url_base(self):
+        return '/accounts/{}/zones'.format(self.account_id)
 
     @property
     def account_id(self):
